
package sub_arctic.input;

import sub_arctic.lib.interactor;

/** 
 * Input protocol for objects wishing to receive drag input suitable for 
 * resizing.  
 *
 * @see sub_arctic.input.grow_drag_focus_agent
 * @author Scott Hudson
 */
public interface grow_draggable extends focusable, interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch the start of a grow-drag to the object (intended to change its
   * size).  Returns true if the drag is accepted (and hence the event 
   * consumed).  The event retains the original raw coordinate values (in 
   * the local coordinates of the object receiving the drag).
   *
   * @param evt the event "causing" the drag
   * @param user_info the information object associated with the drag
   * @return boolean indicating whether the input was consumed/accepted
   */
  public boolean drag_start(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch movement during a grow-drag to the object (intended to change 
   * its size).  Returns true if the drag is accepted (and hence the event 
   * consumed).  The event retains the original raw coordinate values (in 
   * the local coordinates of the object receiving the drag).  Both the 
   * original and new sizes are provided as well.
   *
   * @param evt the event "causing" the movement
   * @param cur_w the current width of the object (based on movement from the 
   *              first drag point)
   * @param cur_h the current height of the object (based on movement from the 
   *              first drag point)
   * @param start_w the original width of the object
   * @param start_h the original height of the object
   * @param user_info the information object associated with the drag
   * @return boolean indicating whether the input was consumed/accepted
   */
  public boolean drag_feedback(
    event evt,     
    int cur_w, int cur_h,
    int start_w, int start_h,
    Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch movement during a grow-drag to the object (intended to change 
   * its size).  Returns true if the drag is accepted (and hence the event 
   * consumed).  The event retains the original raw coordinate values (in 
   * the local coordinates of the object receiving the drag).  Both the 
   * original and new sizes are provided as well.
   *
   * @param evt the event "causing" the movement
   * @param cur_w the current width of the object (based on movement from the 
   *              first drag point)
   * @param cur_h the current height of the object (based on movement from the 
   *              first drag point)
   * @param start_w the original width of the object
   * @param start_h the original height of the object
   * @param user_info the information object associated with the drag
   * @return boolean indicating whether the input was consumed/accepted
   */
  public boolean drag_end(
    event evt,     
    int cur_w, int cur_h,
    int start_w, int start_h,
    Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
