
package sub_arctic.input;

import sub_arctic.lib.interactor;

/** 
 * Input protocol for objects wishing to drag drag into and out of their
 * bounds.  
 *
 * @see sub_arctic.input.inout_drag_agent
 * @author Scott Hudson
 */
public interface inout_draggable extends focusable, interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Dispatch input corresponding to the start of an inout tracking drag. 
   *  The boolean parameter is passed as true if the drag starts inside the 
   *  object.  Returns true if the drag is accepted (and hence the event 
   *  consumed).<p>
   *
   * @param  event    evt           the event associated with the start of drag.
   * @param  boolean  starts_inside does the mouse start inside the object.
   * @param  Object   ui            user info that was passed in when the 
   *                                object was picked.
   */
  public boolean inout_drag_start(
    event    evt, 
    boolean  starts_inside, 
    Object   ui);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Dispatch "drag" into the bounds of the object.  Returns true if the 
   *  input is accepted (and hence the event consumed).<p>
   *
   * @param  event    evt           the event associated with the start of drag.
   * @param  Object   ui            user info that was passed in when the 
   *                                object was picked.
   */
  public boolean inout_drag_enter(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Dispatch "drag" out of the bounds of the object.  Returns true if the 
   *  input is accepted (and hence the event consumed).<p>
   *
   * @param  event    evt           the event associated with the start of drag.
   * @param  Object   ui            user info that was passed in when the 
   *                                object was picked.
   */
  public boolean inout_drag_exit(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Dispatch input corresponding to the end of an inout drag.  The boolean
   *  parameter is passes as true if the drag ended inside the object, false
   *  otherwise.  This routine should return true if the input was accepted
   *  (and hence the event consumed).<p>
   *
   * @param  event    evt           the event associated with the start of drag.
   * @param  boolean  starts_inside does the mouse start inside the object.
   * @param  Object   ui            user info that was passed in when the 
   *                                object was picked.
   */
  public boolean inout_drag_end(
    event   evt, 
    boolean ended_inside, 
    Object  user_info);

   //had:
   //* @exception general 

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}





/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
