package sub_arctic.input;

/**
 * This is an input protocol interface useful for performing actions on the 
 * interface when the mouse enters or leaves an object. <P>
 *
 * <STRONG>Warning</STRONG>: This protocol is based on the various mouse
 * events that are dispatched by the system. If you modify an interface
 * based on other types of events (e.g. animation), the pointable
 * interface's mouse_exit() call will not be made immediately. In particular,
 * such a call will not be made until the next mouse event is processed.
 *
 * @author Ian Smith
 */
public interface pointable {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method is called when the pointer enters you area and
   * when no other agent has dispatched the event.
   * @param event  evt       the event to dispatch
   * @param Object user_info the object you passed to the pick_collector 
   *                         at pick_time
   * @return boolean return true if you dispatched this event
   */
  public boolean mouse_enter(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method is called when the pointer leaves your area. Note that
   * no user_info is given to you, you were not picked by this event!
   * Also, beware that you cannot return a value from this function,
   * because it is not only possible but likely that some other
   * part of the system handled this event.  
   *
   * @param event evt the event to dispatch
   */
  public void mouse_exit(event evt);
  //xx need to do something to make this consistent with other input protocols

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
