
package sub_arctic.input;

/** 
 * Input protocol interface for objects which wish to receive raw events.
 * This protocol is used by both the focus and positional raw dispatch 
 * agents.
 *
 * @see sub_arctic.input.raw_focus_agent
 * @see sub_arctic.input.raw_positional_agent
 * @author Scott Hudson
 */
public interface raw_input_acceptor {

  /** 
   * Dispatch raw input event to the object. 
   * @param event  evt       the event being dispatched.
   * @param Object user_info uninterpreted information that was provided by this
   *                         object either when it became the focus, or for 
   *                         positional dispatch, when the object was picked.
   */
  public boolean handle_raw_input(event evt, Object user_info);

}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
