
package sub_arctic.input;

/** 
 * Input protocol interface for objects which can be put in a currently 
 * selected set (managed by the selection_agent).  
 *
 * @see sub_arctic.input.selection_agent_class
 * @author Scott Hudson
 */
public interface selectable {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch input indicating object has been selected.  If this routine 
   * returns false then the selection is considered to be rejected and the 
   * object is not added to the currently selected object set.
   *
   * @param event  evt       the event "causing" the selection.
   * @param Object user_info the information associated with the pick causing 
   *                         the selection.
   * @return boolean indicating whether the input was consumed.
   */
  public boolean select(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch input indicating object has been unselected.  The object will
   * be removed from the currently selected object set regardless of the 
   * return value from this routine.
   *
   * @param event  evt       the event "causing" the loss of selection.
   * @param Object user_info the information associated with the pick causing 
   *                         the loss of selection.
   * @return boolean indicating whether the input was consumed.
   */
  public boolean unselect(event evt, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
