package sub_arctic.demo_apps;
import sub_arctic.lib.base_parent_interactor;
import sub_arctic.lib.panner;
import sub_arctic.lib.row;
import sub_arctic.lib.button;
import sub_arctic.lib.interactor;
import sub_arctic.lib.manager;
import sub_arctic.lib.display_help_container;
import sub_arctic.lib.text_display;
import sub_arctic.constraints.std_function;
import sub_arctic.output.drawable;
import sub_arctic.output.style;
import sub_arctic.output.style_manager;
import sub_arctic.output.loaded_image;
import sub_arctic.input.pick_collector;
import sub_arctic.input.event;
import sub_arctic.input.move_draggable;
import sub_arctic.input.pointable;
import sub_arctic.input.pressable;
import sub_arctic.input.callback_object;
import sub_arctic.anim.timer;
import sub_arctic.anim.timer_transition;

import java.awt.Font;
import java.applet.Applet;
import java.net.MalformedURLException;
import java.util.Vector;

/**
 * This is a class which can display the various fields of a mail message.
 */

public class message_display extends base_parent_interactor implements move_draggable, callback_object, pressable, timer, pointable {
  /*
   * Icons
   */
  static loaded_image delete_icon = null;
  static loaded_image todo_icon=null;
  static loaded_image goaway_icon=null;
  /*
   * Control button
   */
  button delete;
  button todo;
  button goaway;
  /* the row */
  row button_holder;
  /* our applet */
  mail_test _applet;
  /* our mail message */
  message _msg;

  /**
   * Construct a message display, given a message.
   */
  public message_display(message msg, mail_test applet) {
    field from, to, subject;
    text_display body;
    panner p;

    /* insure that the icons are loaded */
    if (delete_icon==null) {
      load_icons(applet);
    }
    /* copy the applet & message */
    _applet=applet;
    _msg=msg;

    /* ignore the 10x10 size, we fix that with constraints */
    base_parent_interactor field_holder=new base_parent_interactor(0,0,10,10);
    base_parent_interactor body_holder=new base_parent_interactor(0,0,10,10);
    button_holder=new row(2,row.CENTER_JUSTIFIED);

    /* set up the holders right */
    button_holder.set_x(1);
    button_holder.set_y(3);

    field_holder.set_x(3);
    field_holder.set_y_constraint(std_function.offset(PREV_SIBLING.Y2(),3));
    field_holder.set_w_constraint(std_function.eq(PARENT.W()));
    field_holder.set_h_constraint(std_function.offset(LAST_CHILD.Y2(),10));
    
    body_holder.set_x(1);
    body_holder.set_y_constraint(std_function.eq(PREV_SIBLING.Y2()));
    body_holder.set_w_constraint(std_function.eq(PARENT.W()));
    body_holder.set_h_constraint(std_function.fill(PREV_SIBLING.Y2(),
						PARENT.Y2(),0));
    
    add_child(button_holder);
    add_child(field_holder);
    add_child(body_holder);

    /* put in the button children */
    delete=new button(delete_icon,this);
    todo=new button(todo_icon,this);
    goaway=new button(goaway_icon,this);
    button_holder.
      add_child(new display_help_container(todo,"Add to todo list"));
    button_holder.
      add_child(new display_help_container(goaway,
					   "Don't view this message anymore"));
    button_holder.
      add_child(new display_help_container(delete,
					   "Delete message (not implemented)"));

    /* now build the children of the field holder */
    from=new field("From: ",60,msg.from());
    to=new field("To: ",60,msg.to());
    subject=new field("Subject: ",60,msg.subject());
    field_holder.add_child(from);
    field_holder.add_child(to);
    field_holder.add_child(subject);

    /* put the fields in place in y (they take care of the rest)*/
    from.set_y(0);
    to.set_y_constraint(std_function.offset(PREV_SIBLING.Y2(),2));
    subject.set_y_constraint(std_function.offset(PREV_SIBLING.Y2(),2));

    /* put in a text display */
    body=new text_display(0,0,100, 100, /* we are going to force the size */
			  msg.text(),new Font("Courier",8, Font.PLAIN),
			  false);
    body.size_by_content();
    p=new panner(body);
    p.set_x(2);
    p.set_y(2);
    p.set_w_constraint(std_function.offset(PARENT.W(),-6));
    p.set_h_constraint(std_function.offset(PARENT.H(),-6));
    body_holder.add_child(p); 
  }

  /**
   * We want a small border around us, so override and draw the border. We
   * also want to paint in a background so we look opaque.
   */
  public void draw_self_local(drawable d) {
    /* fill with base to make us opaque */
    d.setColor(style_manager.default_color_scheme().base());
    d.fillRect(0,0,w(),h());
    /* set the color back */
    d.setColor(style_manager.default_color_scheme().foreground());
    d.drawRect(0,0,w()-1,h()-1);
    draw_children(d);
  }
  /**
   * How wide is the text? 
   */
  public int text_width()  {
    panner p;
    interactor i;
    p=(panner)child(2).child(0);
    i=p.panned_child();
    return i.w();
  }
  /**
   * How tall is the text? 
   */
  public int text_height()  {
    panner p;
    interactor i;
    p=(panner)child(2).child(0);
    i=p.panned_child();
    return i.h()+ child(1).y() + child(1).h();
  }
  /**
   * Handle press call ... we just use this make the object "feel opaque"
   * mostly. If the press isn't on the magic area, we just destroy it.
   */
  public boolean press(event evt, Object user_info) {
    /* is it in the row?*/
    if (evt.local_y()<=(button_holder.y() + button_holder.h())) {
      /* is it right of the buttons? */
      if (evt.local_x()>(button_holder.child(2).x() + 
			 button_holder.child(2).w())) {
	/* double is in progress? if so, we double click */
	if (double_in_progress) {
	  double_click();
	} else {
	  /* start a move */
	  manager.move_drag_focus.set_focus_to(this,evt,null);
	}
      }
    }
    /* we always swallow the event */
    return true;
  }
  /**
   * We don't bother doing anything in this method, it isn't going
   * to be called except as the second part of a double click and
   * we don't have anything to do in that case.
   */
  public boolean release(event evt, Object user_info) {
    return false;
  }
  /**
   * We need to screen out all picks that would normally go into
   * us so that we only get picked when the mouse is in the
   * row but not on the buttons.
   */
  public void pick(int pt_x, int pt_y, pick_collector pick_list) 
  {
    /* don't pick anything unless we are enabled and its inside our bounds */
    if (enabled() && visible() && picked_by(pt_x, pt_y)) {
      /* do child picks after us so they get the event BEFORE
	 we do... if we get it we want to either move this object
	 or just throw the event away... this makes us "opaque" 
	 with respect to input */
      pick_within_children(pt_x, pt_y, pick_list);
      /* add us after */
      pick_list.report_pick(this);
      
    }
  }
  /**
   * Relocate this message to another point on the screen.
   */
  public void relocate(int x, int y) {
    set_x(x);
    set_y(y);
  }
  /**
   * Start a move drag. We don't use any parameters other than x and y
   */
  public boolean drag_start(event evt, int x, int y, int grab_x, int grab_y,
			    Object user_info) {
    relocate(x,y);
    return true;
  }
  /**
   * Continue a move drag. We don't use any parameters other than x and y
   */
  public boolean drag_feedback(event evt, int x, int y, 
			       int start_x, int start_y,
			       int grab_x, int grab_y,
			       Object user_info) {
    relocate(x,y);
    return true;
  }
  /**
   * End a move drag. We don't use any parameters other than x and y
   */
  public boolean drag_end(event evt, int x, int y, 
			  int start_x, int start_y,
			  int grab_x, int grab_y,
			  Object user_info) {
    relocate(x,y);
    /* is it in the row?*/
    if (evt.local_y()<=(button_holder.y() + button_holder.h())) {
      /* is it right of the buttons? */
      if (evt.local_x()>(button_holder.child(2).x() + 
			 button_holder.child(2).w())) {
	/* we have a possible double in progress */
	double_in_progress=true;
	manager.animation.
	  schedule_transition(new timer_transition(this,350));
      }
    }
    return true;
  }
  /**
   * This gets called when a button gets pushed
   */
  public void callback(interactor from_obj,event evt,
		       int callback_num, Object callback_info) {

    if (from_obj.equals(goaway)) {
      go_away();
    }
    /* deal with putting something in the list */
    if (from_obj.equals(todo)) {
      put_in_todo();
    }
  }
  /**
   * Double click is in progress when this is true.
   */
  public boolean double_in_progress=false;
  /**
   * This gets called when we detected a double click.
   */
  public void double_click() {
    if (child_index()==parent().num_children()-1) {
      move_to_bottom();
    } else {
      move_to_top();
    }
  }
  /**
   * This gets called when the timer expires. We just clear the double
   * click variable.
   */
  public void time_expired(event e /* ignored */) {
    double_in_progress=false;
  }
  /**
   * Make this object get out of its parent
   */
  public void go_away() {
    parent().remove_child(this);
  }
  /**
   * This is a static method to load the three gif files that we
   * are using for the icons at the top of messages.
   */
  protected static void load_icons(Applet applet) {
    try {
      delete_icon=manager.load_doc_image(applet,"./images/delete.gif");
      goaway_icon=manager.load_doc_image(applet,"./images/go_away.gif");
      todo_icon=manager.load_doc_image(applet,"./images/to_do.gif");
    } catch (java.net.MalformedURLException e) {
      applet.showStatus("Problem loading the application images");
    }
  }
  /**
   * Put this message in the todo list.
   */
  public void put_in_todo() {
    message m=_msg.copy(); /* so we don't muck up its state */
    Vector contents;

    /* build a todo item showing the from field */
    m.set_display_field(0);
    todo_list_element le=new todo_list_element(m,_applet,x(), y());
    /* put it in the todo list by adding it to the contents */
    contents=_applet.todo_list.contents_raw(); /* need them as UI components */
    /* add it to the contents */
    contents.addElement(le);
    _applet.todo_list.set_contents(contents);
    /* remove this puppy from the screen */
    parent().remove_child(this);
  }
  /**
   * We need to be opaque with respect to pointable for the objects "behind"
   * us. Since we are "behind" our children in the pick list, we'll get
   * this *after* they do. Since none of our children have handled this
   * event, we just throw it away. We don't want objects behind us getting
   * the event.
   */
  public boolean mouse_enter(event evt, Object user_info) {
    return true; /*fake out stuff behind us */
  } 
  /**
   * This is just here to complete the protocol for pointable 
   */
  public void mouse_exit(event evt) {
    /* do nothing */
  }
  
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
