
package sub_arctic.input;

/** 
 * Input protocol interface for objects which wish to receive a "click" (a 
 * mouse button press and release in a small area).  This protocol is 
 * dispatched by the click_agent dispatch agent.
 *
 * @author Scott Hudson
 */
public interface clickable {
  /** 
   * Dispatch click (press and release in small area) to the object.  Return
   * true if input was consumed.  
   *
   * @param event  evt       the final event in the click sequence.
   * @param Object user_info uninterpreted user information that was passed to
   *                         the pick process that selected this object as a
   *                         candidate for delivery of this event.
   * @return boolean indicating whether the input was consumed.
   */
  public boolean click(event evt, Object  user_info);
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
