
package sub_arctic.input;

/** 
 * Input protocol interface for objects which are to be informed of entry to 
 * and exit from a focus set.  This protocol serves as a base interface which
 * all focus based dispatch protocols inherit from.  Note: because many focus
 * based protocols actually override the use of these methods with something
 * more specific (i.e. they deliver the same inputs using other more specific
 * methods), these methods have been implemented in base_interactor and are 
 * by default ignored.  
 *
 * @author Scott Hudson
 */
public interface focusable {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Inform the object that it has entered the focus set of the given 
   * dispatch agent. <p>
   * 
   * @param event                cause_evt the event that caused the focus.
   * @param focus_dispatch_agent of_agent  the agent doing the focus.
   * @param Object               user_info uninterpreted information that was
   *                                       given to the agent when the focus
   *                                       was established.
   */
  public void focus_set_enter(
    event                cause_evt, 
    focus_dispatch_agent of_agent, 
    Object               user_info);
  
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Inform the object that it has left the focus set of the given 
   * dispatch agent. <p>
   * 
   * @param event                cause_evt the event that caused this.
   * @param focus_dispatch_agent of_agent  the agent doing the this.
   * @param Object               user_info uninterpreted information that was
   *                                       given to the agent when the focus
   *                                       was established.
   */
  public void focus_set_exit(
    event                cause_evt, 
    focus_dispatch_agent of_agent,
    Object               user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
