package sub_arctic.input;

/** 
 * Input protocol for objects receiving input from the simple_press_drag agent.
 * This agent is a hybrid positional-focus.  It responds to press events
 * over an object, by making that object the move_drag focus.  The actual
 * input protocol is the same as simple_drag (i.e., the object never gets or 
 * handles, the press, only the drag). <p>
 *
 * Note: this protocol adds no methods over simple_draggable.  This exists
 * so we can differentiate between objects with want us to handle the press
 * for them, and objects which just want simple_draggable dispatch.<p>
 *
 * @see sub_arctic.input.pressable
 * @see sub_arctic.input.simple_draggable
 * @see sub_arctic.input.simple_press_drag_agent
 *
 * @author Scott Hudson
 */
public interface simple_press_draggable extends simple_draggable {

  /* No extra methods over simple_draggable -- we just need to mark objects 
   * using this via the simple_press_draggable interface name.
   */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
