/*
 * $Id: interactor_consts.java,v 1.8 1996/10/03 19:45:17 hudson Exp $
 * $Author: hudson $
 */

package sub_arctic.lib;

public interface interactor_consts {

  /*---------------------*/
  /* Geometry components */
  /*---------------------*/
  /** Designator bit for x */
  public static final int X = 1<<0;

  /** Designator bit for y */
  public static final int Y = 1<<1;

  /** Designator bit for w */
  public static final int W = 1<<2;

  /** Designator bit for h */
  public static final int H = 1<<3;

  /** Designator bit for visible */
  public static final int VISIBLE = 1<<4;

  /** Designator bit for enabled */
  public static final int ENABLED = 1<<5;

  /** Designator bit for part_a */
  public static final int PART_A = 1<<6;

  /** Designator bit for part_b */
  public static final int PART_B = 1<<7;

  /*----------------------------*/
  /* interactor flag bit values */
  /*----------------------------*/
  /** X position of object is out-of-date */
  public static final int X_OOD                  = 1<<0; 

  /** Y position of object is out-of-date */
  public static final int Y_OOD                  = 1<<1;  

  /** Width of object is out-of-date */
  public static final int W_OOD                  = 1<<2; 

  /** Height of object is out-of-date */
  public static final int H_OOD                  = 1<<3;

  /** Height of object is out-of-date */
  public static final int VISIBLE_OOD            = 1<<4;

  /** Height of object is out-of-date */
  public static final int ENABLED_OOD            = 1<<5;

  /** Height of object is out-of-date */
  public static final int PART_A_OOD             = 1<<6;

  /** Height of object is out-of-date */
  public static final int PART_B_OOD             = 1<<7;

  /** All out-of-date bits */
  public static final int OOD_BITS = 
     X_OOD|Y_OOD|W_OOD|H_OOD|VISIBLE_OOD|ENABLED_OOD|PART_A_OOD|PART_B_OOD;

  /** X has external entities to notify when it goes out-of-date */ 
  public static final int X_HAS_EXT_NOTIFY       = 1<<8;

  /** Y has external entities to notify when it goes out-of-date */ 
  public static final int Y_HAS_EXT_NOTIFY       = 1<<9;

  /** W has external entities to notify when it goes out-of-date */ 
  public static final int W_HAS_EXT_NOTIFY       = 1<<10;

  /** H has external entities to notify when it goes out-of-date */ 
  public static final int H_HAS_EXT_NOTIFY       = 1<<11;

  /** VISIBLE has external entities to notify when it goes out-of-date */ 
  public static final int VISIBLE_HAS_EXT_NOTIFY = 1<<12;

  /** ENABLED has external entities to notify when it goes out-of-date */ 
  public static final int ENABLED_HAS_EXT_NOTIFY = 1<<13;

  /** PART_A has external entities to notify when it goes out-of-date */ 
  public static final int PART_A_HAS_EXT_NOTIFY   = 1<<14;

  /** PART_B has external entities to notify when it goes out-of-date */ 
  public static final int PART_B_HAS_EXT_NOTIFY   = 1<<15;

  /** Evaluation is underway on X (used for cycle detection). */
  public static final int X_IN_PROCESS           = 1<<16;

  /** Evaluation is underway on Y (used for cycle detection). */
  public static final int Y_IN_PROCESS           = 1<<17;

  /** Evaluation is underway on W (used for cycle detection). */
  public static final int W_IN_PROCESS           = 1<<18;

  /** Evaluation is underway on H (used for cycle detection). */
  public static final int H_IN_PROCESS           = 1<<19;

  /** Evaluation is underway on VISIBLE (used for cycle detection). */
  public static final int VISIBLE_IN_PROCESS      = 1<<20;

  /** Evaluation is underway on ENABLED (used for cycle detection). */
  public static final int ENABLED_IN_PROCESS      = 1<<21;

  /** Evaluation is underway on PART_A (used for cycle detection). */
  public static final int PART_A_IN_PROCESS        = 1<<22;

  /** Evaluation is underway on PART_B (used for cycle detection). */
  public static final int PART_B_IN_PROCESS        = 1<<23;

  /** Indicate whether VISIBLE is consrainted to a horizontal value. */
  public static final int VISIBLE_IS_HORIZONTAL   = 1<<24;

  /** Indicate whether ENABLED is consrainted to a horizontal value. */
  public static final int ENABLED_IS_HORIZONTAL   = 1<<25;

  /** Indicate whether PART_A is consrainted to a horizontal value. */
  public static final int PART_A_IS_HORIZONTAL     = 1<<26;

  /** Indicate whether PART_B is consrainted to a horizontal value. */
  public static final int PART_B_IS_HORIZONTAL     = 1<<27;

  /* Note: when a new constant is added here be sure to update 
   * base_interactor.flag_string().  
   */

  /*----------------------------*/
  /* interactor flag bit values */
  /*----------------------------*/
  /** Object is marked as visible */
  public static final int IS_VISIBLE         = 1<<0;

  /** Object is marked as enabled (accepting input) */
  public static final int IS_ENABLED         = 1<<1;

  /** Object can be a parent */
  public static final int SUPPORTS_CHILDREN  = 1<<2;

  /** Object uses fixed children */
  public static final int FIXED_CHILDREN     = 1<<3;

  /** Object screen apperance is damaged and needs redraw */
  public static final int DAMAGED            = 1<<4;

  /* Note: when a new constant is added here be sure to update 
   * base_interactor.flag_string().  
   */

  /** First flag bit not assigned at the interactor level.  Bits starting with
   *  this one are free to be used bye subclasses for special purposes.  
   */
  public static final int FIRST_FREE_FLAG    = 1<<5;

  /*-------------------------*/
  /* standard feature points */
  /*-------------------------*/

  /** Indicator for no feature point.  This is usually used in drags and snaps 
   *  to indicate that the alignment point should be the point at which the 
   *  object was "picked up" rather than a feature point of the object.
   */
  public static final int FEATURE_NONE = -1;

  /** Feature point index for top-left corner */
  public static final int FEATURE_TOP_LEFT = 0;

  /** Feature point index for top-right corner */
  public static final int FEATURE_TOP_RIGHT = 1;

  /** Feature point index for bottom-left corner */
  public static final int FEATURE_BOTTOM_LEFT = 2;

  /** Feature point index for bottom-right corner */
  public static final int FEATURE_BOTTOM_RIGHT = 3;

  /** Feature point index for center of object */
  public static final int FEATURE_CENTER = 4;

  /** First feature point index available for use by interactor subclasses. */
  public static final int FIRST_FREE_FEATURE = 5;

  /*------------------*/
  /* traversal orders */
  /*------------------*/

  /** Indicator for a traversal in drawing order.  Default drawing order is
   *  left-to-right post-order, however, subclasses can change this order.
   */
  public static final int TRAV_DRAW = 0;

  /** Indicator for a traversal in picking order.  Picking order should 
   *  be the reverse of drawing order.  This would default to a right-to-left
   *  pre-order traversal, however, subclasses can override this.
   */
  public static final int TRAV_PICK = 1;

  /** Indicator for a left-to-right pre-order traversal. */
  public static final int TRAV_PRE  = 2;

  /** Indicator for a left-to-right post-order traversal. */
  public static final int TRAV_POST  = 3;
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
