package sub_arctic.lib;

/** 
 * Interface representing an object that tests a predicate over an interactor
 * object (with a set of subclass specific parameter values).  This is used
 * as part of the traverse_and_collect() traversal method described in 
 * base_interactor.<p>  
 * 
 * In addition to implementing a predicate, this class is also sometimes used
 * to perform actions during a traversal.  In that case the test() method is
 * executed for its side effects in addition to its boolean selection result.
 *
 * @see sub_arctic.lib.base_interactor#traverse_and_collect
 * @author Scott Hudson
 */
public interface interactor_pred {
  /** Perform the predicate test.
   *  
   *  @param obj        the interactor the predicate is testing.
   *  @param parameters the additional parameters (of subclass specific type)
   *                    to the predicate.
   *  @return predicate result.
   */
  public boolean test(interactor obj, Object parameters);

   //had:
   //*  @exception sub_arctic.exception.bad_value thrown if the parameters 
   //*     parameter is not the proper type.
};
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
