package sub_arctic.lib;

import sub_arctic.input.callback_object;
import sub_arctic.input.event;
import sub_arctic.input.pick_collector;
import sub_arctic.output.style_manager;
import sub_arctic.output.color_scheme;
import sub_arctic.output.style;
import sub_arctic.output.drawable;
import sub_arctic.output.loaded_image;
import sub_arctic.constraints.std_function;

import java.util.Vector;

/**
 * This is the class that handles the output side of the behavior of
 * a listbox. It has no input behavior of its own.
 * 
 * @author Ian Smith
 */
public class listbox_display extends base_parent_interactor implements callback_object {
  /**
   * The underlying listbox_display_children.
   */
  protected listbox_child_display _child_display;
  /**
   * The parent interactor which holds the listbox display and the horiz
   * scrollbar.
   */
  protected listbox_helper _display_parent;
  /**
   * This is the horizontal scrollbar for this listbox (which 
   * is created even if not used.
   */
  protected h_scrollbar _horiz;
  /**
   * This is the vertical scrollbar for this listbox (which 
   * is created even if not used).
   */
  protected v_scrollbar _vert;
  /**
   * This is an "external" vertical scrollbar. This is set to a value
   * by the user if he wishes to have an external scrollbar control
   * the listbox.
   */
  protected v_scrollbar _external_v_scrollbar=null;
  /**
   * This is an "external" horizontal scrollbar. This is set to a value
   * by the user if he wishes to have an external scrollbar control
   * the listbox.
   */
  protected h_scrollbar _external_h_scrollbar=null;
  /**
   * This function gets called to set up the heirarchy of interactors
   * that a listbox needs. 
   */
  protected void setup_children() {
    /* we assume all the objects are created by this point */
    /* we are going to make a backdrop of the right color */
    loaded_image pattern=new loaded_image(100,100);
    drawable d;
    backdrop drop;
    style cs=style_manager.current_style();
    int dvs=cs.drawable_vertical_space();
    int dhs=cs.drawable_horizontal_space();

    /* set the drawable to the right color then a fill the rect */
    d=pattern.get_drawable();
    d.setColor(style_manager.default_color_scheme().text_background());
    d.fillRect(0,0,100,100);
    /* make a backdrop */
    drop=new backdrop(pattern);

    /* if this object already has children, we can safely just get rid
     * of them 
     */
    while (num_children()>0) {
      remove_child(0);
    }

    /* put the parent in */
    add_child(_display_parent);
    /* its location is always at 0,0 */
    _display_parent.set_x(0);
    _display_parent.set_y(0);
    /* its constraint is FILL in the X dimension */
    _display_parent.
      //set_w_constraint(std_function.fill(PARENT.X(), NEXT_SIBLING.X(),0));
      set_w_constraint(std_function.fill(PARENT.X(), PARENT.X2(),0));
      
    /* it covers all of this parent in Y */
    _display_parent.
      set_h_constraint(std_function.offset(PARENT.H(),0));
    /* 
     * the three children of this parent are the horiz scrollbar,
     * the display, and the backdrop... 
     */
    /* put the backdrop in first so it is behind... it fills the same
     exact space as the list */
    drop.set_w_constraint(std_function.offset(NEXT_SIBLING.W(),0));
    drop.set_h_constraint(std_function.offset(NEXT_SIBLING.H(),0));
    /* needs to fill the available space of the objects */
    drop.set_x_constraint(std_function.offset(NEXT_SIBLING.X(),0));
    drop.set_y_constraint(std_function.offset(NEXT_SIBLING.Y(),0));
    /* add the backdrop */
    _display_parent.add_child(drop);
    /* now the display */
    _display_parent.add_child(_child_display);
    /* we don't bother making the child be at a particular point,
       since the scrolling code makes this work 
       */
    /* we just make the child display shrink wrap in Y*/
    _child_display.set_h_constraint(std_function.offset(LAST_CHILD.Y2(),0));
    /* this object computes its own width, we don't deal with that */
    /* 
     * we might or might not need scrollbars, lets deal with them
     * as well as consider the sizes of the thumbs if necessary 
     */
    reconfigure_children();
  }
  /**
   * This is the constant which expresses that you want scrollbars
   * only when they are necessary.
   */
  public static final int SCROLLBAR_AS_NECESSARY=0;
  /**
   * This is the constant which expresses that you always want
   * a scrollbar, even if the display doesn't fill the
   * object.
   */
  public static final int SCROLLBAR_ALWAYS=1;
  /**
   * This is the constant which expresses that you never want
   * a scrollbar, no matter what the size of the list. This 
   * presumes you are going to handle the scrolling yourself.
   */
  public static final int SCROLLBAR_NEVER=2;
  /**
   * This holds the status of the vertical scrollbar.
   */
  protected int _vert_status=SCROLLBAR_AS_NECESSARY;
  /**
   * Retrieve the status of the vertical scrollbar. This status
   * will be one of SCROLLBAR_AS_NECESSARY, SCROLLBAR_ALWAYS, 
   * or SCROLLBAR_NEVER.
   */
  public int vertical_status() {
    return _vert_status;
  }
  /**
   * Retrieve the status of the horizontal scrollbar. This status
   * will be one of SCROLLBAR_AS_NECESSARY, SCROLLBAR_ALWAYS, 
   * or SCROLLBAR_NEVER.
   */
  public int horizontal_status() {
    return _horiz_status;
  }
  /**
   * This holds the status of the horizontal scrollbar.
   */
  protected int _horiz_status=SCROLLBAR_AS_NECESSARY;
  /**
   * Set the status of the vertical scrollbar. The value passed
   * to this function must be one of the constants
   * SCROLLBAR_AS_NECESSARY, SCROLLBAR_ALWAYS, or SCROLLBAR_NEVER.
   */
  public void set_vertical_status(int status) {
    /*short circuit */
    if (status==_vert_status) return;

    switch (status) {
    case 1:
    case 2:
      _vert_status=status;
      break;
    case 0:
    default:
      _vert_status=SCROLLBAR_AS_NECESSARY;
    }
    /* update the scrollbars */
    reconfigure_children();
  }
  /**
   * Set the status of the horizontal scrollbar. The value passed
   * to this function must be one of the constants
   * SCROLLBAR_AS_NECESSARY, SCROLLBAR_ALWAYS, or SCROLLBAR_NEVER.
   */
  public void set_horizontal_status(int status) {
    /*short circuit */
    if (status==_horiz_status) return;

    switch (status) {
    case 1:
    case 2:
      _horiz_status=status;
      break;
    case 0:
    default:
      _horiz_status=SCROLLBAR_AS_NECESSARY;
    }
    /* update the scrollbars */
    reconfigure_children();
  }
  /**
   * Create a new listbox display. It defaults to adding scrollbars on
   * demand. It also defaults to a location of 0,0 so you must use
   * constraints or position this object yourself. Its size will
   * be 150x100.
   *
   */
  public listbox_display() {
    this(150,100); // default size of 150x100
  }
  /**
   * Create a new listbox display with a given size. It defaults to
   * adding scrollbars on demand; it sets its position to 0, 0
   * so either position it with constraints or set the coordinates
   * directly.
   * @param int w the width of the interactor
   * @param int h the height of the interactor
   */
  public listbox_display(int w, int h)  {
    super(0,0,w,h);
    /*
     *  make the child objects  ...all of these objects sizes and
     *   and positions will be dealt with via constraints later
     */
    /* first the object to hold the children */
    _child_display=new listbox_child_display(10,10,this);
    /* now the object that holds the child_display and the horiz 
       scrollbar */
    _display_parent=new listbox_helper();
    /* now the two scrollbars */
    _vert=new listbox_v_scrollbar(this);
    _horiz=new h_scrollbar(this);
    /* build the heirarchy and set the constraints */
    setup_children(); 
  }
  /**
   * This method is called by our child to tell us the size has changed.
   */
  public void child_size_changed() {
    /* call the reconfigure code to deal with scrollbar sizes and
       whatnot */
    reconfigure_children();
  }
  /**
   * We override configure so we can become informed of our size
   * changing and do the resizing of the displayed objects if necessary.
   */
  public void configure() {
    super.configure();
    _child_display.calculate_width();
    ensure_thumb_size();
    /*
      reconfigure_children();
    */
  }
  /**
   * Clear the set of objects that are displayed.
   */
  public void clear() {
    _child_display.set_contents(null);
  }
  /**
   * Set the objects that are displayed. This vector must contain
   * only  list_element objects.
   *
   * @param Vector contents the objects to display
   */
  public void set_contents(Vector contents) {
    _child_display.set_contents(contents);
    /* no selections yet */
    set_selected(null);
  }
  /**
   * Retreive the contents of the listbox transformed by
   * the listbox elements via convert_to_object.
   * If the listbox is empty, this will return an empty Vector.
   * @return Vector the contents of the listbox
   */
  public Vector contents() {
    return _child_display.contents();
  }
  /**
   * Retreive the contents of the listbox as list_elements.
   * If the listbox is empty, this will return an empty Vector.
   * @return Vector the contents of the listbox
   */
  public Vector contents_raw() {
    return _child_display.contents_raw();
  }
  /**
   * Set the selected set of objects. This vector must contain
   * only list_element objects or it can be null to clear all
   * selections.
   *
   * @param Vector selected the objects which are now to be selected
   */
  public void set_selected(Vector selected) {
    _child_display.set_selected(selected);
  }
  /**
   * Add objects to the selected set. This takes two integers which are
   * object numbers of the objects you wish to be selected. If these 
   * objects are already selected, this does not select them again.
   * @param int start the first object to be added
   * @param int end the last object to be added  (inclusive)
   */
  public void add_to_selected_set(int start, int end)  {
    list_element elem;
    int i,tmp;
    Vector selected_set=selected_set_raw();
    Vector new_elem=new Vector();
    /* canonicalize the order */
    if (start>end) {
      tmp=start;
      start=end;
      end=tmp;
    }
    if (start<0) System.out.println("WARNING: Start is " + start);
    if (end<0) System.out.println("WARNING: end is " + end);
    /* loop over what they gave us */
    for (i=start; i<=end; ++i) {
      elem=(list_element)_child_display.child(i);
      /* is it already selected?*/
      if (!selected_set.contains(elem)) {
	/* not selected */
	new_elem.addElement(elem);
      }
    }
    /* now new_elem has all the objects which are not yet selected */
    for (i=0; i<new_elem.size() ; ++i) {
      /* put in the set */
      selected_set.addElement(new_elem.elementAt(i));
    }
    /* reset the set */
    if (new_elem.size()!=0) {
      set_selected(selected_set);
    }
  }
  /**
   * Remove a items from the selected set based on their item number. 
   * If the item isn't in the selected set, it doesn't do anything.
   *
   * @param int start the first item to remove
   * @Param int end the last item to remove
   */
  public void remove_from_selected_set(int start, int end)  {
    list_element elem;
    int i,tmp;
    Vector selected_set=selected_set_raw();
    Vector old_elem=new Vector();
    /* canonicalize the order */
    if (start>end) {
      tmp=start;
      start=end;
      end=tmp;
    }
    /* loop over what they gave us */
    for (i=start; i<=end; ++i) {
      elem=(list_element)_child_display.child(i);
      /* is it already selected?*/
      if (selected_set.contains(elem)) {
	/* it is selected */
	old_elem.addElement(elem);
      }
    }
    /* now new_elem has all the objects which need to go */
    for (i=0; i<old_elem.size() ; ++i) {
      /* put in the set */
      selected_set.removeElement(old_elem.elementAt(i));
    }
    /* reset the set */
    if (old_elem.size()!=0) {
      set_selected(selected_set);
    }
  }
  /**
   * Clear the set of selected items.
   */
  public void clear_selected() {
    set_selected(null);
  }
  /**
   * This is the callback object code. We use this to get the information
   * from the scrollbars.
   * @param interactor from_obj the source of the callback
   * @param event evt the event causing the callback 
   * @param int callback_num the callback number 
   * @param Object callback_info the callback supplied information
   */
  public void callback(interactor from_obj,event evt, int callback_num,
		       Object callback_info)  {
    int foo;
    //its always an integer from the scrollbars
    Integer info=(Integer)callback_info;

    /* is it from the horizontal scrollbar */
    if (from_obj.equals(horizontal_scrollbar())) {
      set_scrolled_x(- (info.intValue()));
    }
    /* vertical scrollbar? */
    if (from_obj.equals(vertical_scrollbar())) {
      set_scrolled_y(info.intValue());
    }
  }
  /**
   * This function is the one that does the work of deciding if we need
   * the scrollbars and if so how big the thumbs of the scrollbars 
   * should be.
   */
  protected void reconfigure_children() {
    boolean need_v_scrollbar=false,need_h_scrollbar=false;
    int v_height_offset, available_height, available_width, last_obj=0;
    style cs=style_manager.current_style();
    int dhs=cs.drawable_horizontal_space(),last_object;

    /* 
     * Phase 1: Decide if we need h or v scrollbars 
     */
    /* remove either scrollbar if they are there, so we can do the
     * calculations without considering them */
    _display_parent.remove_child(_horiz);
    remove_child(_vert);
    /* now calculate the usable area of the display parent 
     ... this calculation does not account for scrollbars, however */
    available_width=_display_parent.available_width();
    available_height=_display_parent.available_height();

    /* vertical: if it is always, just set the value */
    if (_vert_status==SCROLLBAR_ALWAYS) {
      need_v_scrollbar=true;
    } else {
      /* is it on demand, and necessary? */
      if ((_vert_status==SCROLLBAR_AS_NECESSARY) &&
	  !_child_display.will_fit_height(available_height)) {
	/* its too tall, we need one */
	need_v_scrollbar=true;
      }
    }
    /* horizontal: if it is always, just set the value */
    if (_horiz_status==SCROLLBAR_ALWAYS) {
      need_h_scrollbar=true;
    } else {
      /* is it on demand, and necessary? */
      if ((_horiz_status==SCROLLBAR_AS_NECESSARY) &&
	  !_child_display.will_fit_width(available_width)) {
	/* its too wide, we need one */
	need_h_scrollbar=true;
      }
    }
    /* 
     * tricky case: if we have only one of the two scrollbars,
     * we might need the other because of the space now consumed
     * by the existing scrollbar.
     */
    /* do we have only one scrollbar? */
    if (((need_h_scrollbar) && (!need_v_scrollbar)) ||
	(!need_h_scrollbar) && (need_v_scrollbar)) {
      /* possible ugly case if they are on demand */
      if ((!need_v_scrollbar) && (_vert_status==SCROLLBAR_AS_NECESSARY)) {
	/* ugly case occurs when the horiz scrollbar's height
	   is enough to force a need for this one */
	if (!_child_display.will_fit_height(available_height-_horiz.h())) {
	  /* need it */
	  need_v_scrollbar=true;
	}
      }
      /* this is the other ugly case in the other dimension */
      if ((!need_h_scrollbar) && (_horiz_status==SCROLLBAR_AS_NECESSARY)) {
	/* did the vertical scrollbar contribute enough to hose us? */
	if (!_child_display.will_fit_width(available_width-_vert.w())) {
	  /* we got hosed, we need the scrollbar */
	  need_h_scrollbar=true;
	}
      }
    }
    /*
     * Phase 2: We now know that we have the scrollbar variables set 
     * up so lets put the right scrollbars in the right places 
     */
    if (need_h_scrollbar) {
      _display_parent.add_child(_horiz);
      /* all the way across the parent */
      _horiz.set_w_constraint(std_function.
			      offset(PARENT.W(),
				     -(_display_parent._left_space+
				       _display_parent._right_space)));
      /* against the bottom of the parent */
      _horiz.set_y_constraint(std_function.far_edge_just(PARENT.Y2(),0));
      /* flush with left */
      _horiz.set_x_constraint(std_function.
			      offset(ZERO,_display_parent._left_space));
    }
    /* deal with vertical */
    if (need_v_scrollbar) {
      add_child(_vert);
      /* should go from top of parent to the top of the h scrollbar
	 if we have one */
      if (need_h_scrollbar) {
	v_height_offset=_horiz.h() + _display_parent._above_space + 
	  _display_parent._below_space;
	_vert.set_h_constraint(std_function.offset(PARENT.H(),-v_height_offset));
      } else {
	/* just got all the way to bottom*/
	v_height_offset=_display_parent._above_space + 
	  _display_parent._below_space;
	_vert.set_h_constraint(std_function.offset(PARENT.H(),-v_height_offset));
      }
      /* position this scrollbar against our right side */
      _vert.set_x_constraint(std_function.far_edge_just(PARENT.X2(),0));
      _vert.set_y_constraint(std_function.
			     offset(ZERO, _display_parent._above_space));
    }
    /*
     * Phase 3: set the thumbs to the right size and reset the origin
     */
    ensure_thumb_size();
    vertical_scrollbar().set_value(0);
    horizontal_scrollbar().set_value(0);
    /* set it to be at logical position 0 in X*/
    set_scrolled_x(0);
    /* same for y */
    set_scrolled_y(0);
  }
  /**
   * This function will move the scrolled object (the display of objects)
   * to a particular <I>logical</I> point in x. This function does the
   * work of dealing with moving the scrolled to the correct point
   * which is not the same as the input point due to margins and bevelling.
   *
   * @param int x the new x position of the scrolled object
   */
  public void set_scrolled_x(int x) {
    style cs=style_manager.current_style();
    int dhs=cs.drawable_horizontal_space();

    _child_display.set_x(x+dhs+_display_parent._left_space);
  }
  /**
   * This function will move the scrolled object in such a way that 
   * the parameter given will be at the top of the displayed area.<P>
   *
   * It takes into account margins and bevelling. 
   * @param int y in the index of the object to display
   */
  public void set_scrolled_y(int y) {
    style cs=style_manager.current_style();
    int dvs=cs.drawable_vertical_space(),offset;
    
    offset=_display_parent._above_space+ dvs;
    _child_display.set_y(offset-y);
      return;
  }
  /**
   * This function gets called (via the special scrollbar) when
   * the user wishes to move to the next item.
   */
  protected void move_down() {
    style cs=style_manager.current_style();
    int dvs=cs.drawable_vertical_space(),offset,index;
    list_element i;

    /* short circuit out of here if empty */
    if (_child_display.num_children()==0) return;
    /* get the offset */
    offset=_display_parent._above_space+ dvs;
    
    /* figure out the child number at the top of box */
    index=_child_display.index_for_pixel(offset - (_child_display.y()));
    /* get the element */
    i=(list_element)_child_display.child(index);
    /* we can try to move down */
    index++;
    /* is that in bounds? */
    if (index>=_child_display.num_children()) {
      /* out of bounds, just the value to the maximum value in case
	 the last item is larger than the displayed in area */
      _vert.set_value(_vert.max_val());
    } else {
      /* just set the scrollbar value and let the callback handle it */
      /* note that this might be an "out of bounds" value with respect
	 to the scrollbar. Its "conform()" function will put it back
	 in bounds if that is necessary. Further (to make life easy) it
	 will set the value to the maximum value, so we end up getting
	 a "scroll to bottom" function for nothing
	 */
      _vert.set_value(_child_display.child(index).y());
    }
    return;
  }
  /**
   * This function gets called (via the special scrollbar) when
   * the user wishes to move to the previous item.
   */
  protected void move_up() {
    style cs=style_manager.current_style();
    int dvs=cs.drawable_vertical_space(),offset,index;
    list_element i;

    /* short circuit out of here if empty */
    if (_child_display.num_children()==0) return;
    /* get the offset */
    offset=_display_parent._above_space+ dvs;
    
    /* figure out the child number at the top of box */
    index=_child_display.index_for_pixel(offset - (_child_display.y()));
    /* get the element */
    i=(list_element)_child_display.child(index);
    /* is it exact? */
    if (i.y()==offset-(_child_display.y())) {
      /* are they at the top?*/
      if (index==0) {
	/* nothing we can do */
	return;
      }
      /* we can move up */
      index--;
      /* just set the scrollbar value and let the callback handle it */
      _vert.set_value(_child_display.child(index).y());
      return;
    } else {
      /* we know what we want is the top of this object so just
	   set the scrollbar to that value and let it do the work */
	/* just set the scrollbar value and let the callback handle it */
      _vert.set_value(_child_display.child(index).y());
      return;
    }
  }
  /**
   * This function retreives the currently selected set of objects
   * as list_elements. This is primarily useful if you wish to address
   * these objects as interactors, rather than as data values.
   * @return Vector a vector of list_elements which are those objects that are currently selected. This will return an empty (not null) Vector if the selected set is currently empty
   */
  public Vector selected_set_raw() {
    return _child_display.selected_set_raw();
  }
  /**
   * This function retreives the currently selected set of objects
   * as values. list_element objects which support the 
   * "convert_to_object" method will return a more convenient 
   * representation of themselves, and those that done will be returned
   * as list_elements. If nothing is selected, this will return an
   * empty (not null) vector.
   *
   * @return Vector a vector of selected set objects
   */
  public Vector selected_set() {
    return _child_display.selected_set();
  }
  /**
   * This function makes sure that a given index position of the
   * selected_set is visible on the screen.
   * @param int target the object's index to make visible
   */
  public void ensure_visible(int target) {
    style cs=style_manager.current_style();
    int obj_at_top, obj_at_bot, available_height;
    int dvs=cs.drawable_vertical_space(),offset;
    list_element obj;

    /* compute the offset from the top of the parent*/
    offset=_display_parent._above_space+ dvs;
    /* how much space is there? */
    available_height=_display_parent.available_height();
    /* do they have a horizontal scrollbar? */
    if (_display_parent.has_h_scrollbar()) {
      /* they do, subtract that off */
      available_height-=_horiz.h();
    }
    /* set up our object */
    obj=(list_element)_child_display.child(target);
    /* initialize the to and bottom */
    obj_at_top=_child_display.index_for_pixel(offset - 
					      (_child_display.y()));
    obj_at_bot=_child_display.index_for_pixel((offset - 
					       (_child_display.y()))+
					      available_height);

    /* XXX Should be checking here to make sure that if the
	 target==obj_at_top  or target==obj_at_bot and the
	 top or bottom object should be checked to see if it is
	 fully visible. If it is do nothing, if it isn't then
	 we should make sure it is. If you naively just use
	 <= and >= below the listbox won't work if the objects
	 ARE fully visible XXX */

      /* are they above the top?*/
      /* note: the <= is so that partially visible objects become
	 fully visible */
    if (target<obj_at_top) {
      /* they are so make that object's y position where we go */
      vertical_scrollbar().set_value(obj.y());
      set_scrolled_y(obj.y());
    } else {
      /* are they off the bottom? */
      /* as above we >= so partially visible objects become fully visible */
      if (target>obj_at_bot) {
	vertical_scrollbar().set_value((obj.y() + obj.h())-available_height);
	set_scrolled_y((obj.y() + obj.h())-available_height);
      }
    }
  }
  /**
   * This functions sets the selected set of objects by their 
   * index numbers.
   * @param Vector selected a vector of Integer objects
   */
  public void set_selected_by_index(Vector selected) {
    int val,i;
    Integer tmp;
    Vector contents=contents_raw(),result=new Vector();
    /* loop over all the values */
    for (i=0; i<selected.size(); ++i) {
      /* retreive the int */
      tmp=(Integer)selected.elementAt(i);
      /* grab the value */
      result.addElement(contents.elementAt(tmp.intValue()));
    }
    /* now make those the selected items */
    _child_display.set_selected(result);
  }
  /**
   * This function returns the current vertical scrollbar for
   * the purposes of callback handling and the setting of the
   * scrollbar thumb. This always returns a scrollbar,
   * even if no scrollbar is visible. Also, this should only
   * be used when one is considering manipulating the size
   * of the thumb, the minimum/maximum values, and handling
   * callbacks as it may be a user scrollbar somewhere on the
   * screen other than connected to the listbox.
   * @return v_scrollbar the h_scrollbar in use
   */
  public v_scrollbar vertical_scrollbar() {
    if (_external_v_scrollbar!=null) {
      return _external_v_scrollbar;
    }
    return _vert;
  }
  /**
   * This function returns the current horizontal scrollbar for
   * the purposes of callback handling and the setting of the
   * scrollbar thumb. This always returns a scrollbar,
   * even if no scrollbar is visible. Also, this should only
   * be used when one is considering manipulating the size
   * of the thumb, the minimum/maximum values, and handling
   * callbacks as it may be a user scrollbar somewhere on the
   * screen other than connected to the listbox.
   * @return h_scrollbar the h_scrollbar in use
   */
  public h_scrollbar horizontal_scrollbar() {
    if (_external_h_scrollbar!=null) {
      return _external_h_scrollbar;

    }
    return _horiz;
  }
  /**
   * This function sets the external vertical scrollbar. If you
   * use this function, the scrollbar supplied here will control
   * this listbox's vertical position. You will probably want to
   * use a listbox_v_scrollbar rather than a normal v_scrollbar
   * for this. You should pass null to reset this object to
   * not using an external scrollbar.
   *
   * @param v_scrollbar n the new vertical scrollbar
   */
  public void set_external_v_scrollbar(v_scrollbar n) {
    _external_v_scrollbar=n;
    reconfigure_children();
  }
  /**
   * This function sets the external horizontal scrollbar. If you
   * use this function, the scrollbar supplied here will control
   * this listbox's horizontal position. 
   *
   * @param h_scrollbar n the new horizontal scrollbar
   */
  public void set_external_h_scrollbar(h_scrollbar n) {
    _external_h_scrollbar=n;
    reconfigure_children();
  }
  /**
   * Retrieve the right hand spacing around a listbox.
   * @return int the number of pixels drawn to the right of a listboxes edge
   */
  public int right_space() {
    return _display_parent._right_space;
  }
  /**
   * Retrieve the left hand spacing around a listbox.
   * @return int the number of pixels drawn to the left of a listboxes edge
   */
  public int left_space() {
    return _display_parent._left_space;
  }
  /**
   * Retrieve the above spacing around a listbox.
   * @return int the number of pixels drawn above a listboxes 
   */
  public int above_space() {
    return _display_parent._above_space;
  }
  /**
   * Retrieve the below spacing around a listbox.
   * @return int the number of pixels drawn to below a listbox
   */
  public int below_space() {
    return _display_parent._below_space;
  }
  /**
   * Set the right hand spacing around a listbox.
   * @param int v the new value for the spacing
   */
  public void set_right_space(int v) {
    _display_parent._right_space=v;
    damage_self();
  }
  /**
   * Set the left hand spacing around a listbox.
   * @param int v the new value for the spacing
   */
  public void set_left_space(int v) {
    _display_parent._left_space=v;
    damage_self();
  }
  /**
   * Set the above spacing around a listbox.
   * @param int v the new value for the spacing
   */
  public void set_above_space(int v) {
    _display_parent._above_space=v;
    damage_self();
  }
  /**
   * Set the below spacing around a listbox.
   * @param int v the new value for the spacing
   */
  public void set_below_space(int v) {
    _display_parent._below_space=v;
    damage_self();
  }
  /**
   * This function is called to ensure that the size displayed by the
   * scrollbar thumb(s) is correct. This also updates the min, max, and
   * current values of the thumb(s).
   */
  public void ensure_thumb_size() {
    h_scrollbar hs;
    v_scrollbar vs;
    double percentage;
    int max;
    style cs=style_manager.current_style();
    int dhs=cs.drawable_horizontal_space();

    /* this is the usable area of the display parent, not considering
       the scrollbars  */
    int available_width=_display_parent.available_width();
    int available_height=_display_parent.available_height();

    /* deal with the horizontal scrollbar */
    if (_display_parent.has_h_scrollbar() || (_external_h_scrollbar!=null)) {
      /* get the scrollbar they want us to set the values on */
      hs=horizontal_scrollbar();
      /* if they have a vertical, there is less space */
      if (have_v_scrollbar()) {
	available_width-=_vert.w();
      }
      /**
       * IMPORTANT IMPORTANT IMPORTANT
       * We change the values of the scrollbars parameters from
       * inside configure. This can lead to an infinite loop if 
       * we don't check to make sure we aren't changing the values.
       */
      /* set the horizontal's size based on the number of pixels */
      if (hs.min_val()!=0)
	hs.set_min_val(0);
      /* max is: width - available_width -> to get the right edges lined up
                + left_spacing  -> pixels lost to left hand border shift
		+ dhs           -> pixels lost to left hand bevel shift 
		*/
      max=(_child_display.w()-available_width) + _display_parent._left_space+
	dhs;

      if (hs.max_val()!=max) 
	hs.set_max_val(max);
      if (hs.large_inc()!=available_width)
	hs.set_large_inc(available_width);
      if (hs.small_inc()!=available_width/5)
	hs.set_small_inc(available_width/5);
      /* compute the thumb size */
      percentage=((double)available_width)/((double)_child_display.w());
      if (percentage>1.0) {
	percentage=1.0;
      }
      if (hs.thumb_percentage()!=percentage) 
	hs.set_thumb_percentage(percentage);
    }
    /* deal with vertical scrollbar */
    if (have_v_scrollbar() || (_external_v_scrollbar!=null)) {
      /* get the vertical one */
      vs=vertical_scrollbar();
      /* if we have a vertical, we need to reevaluate the available height */
      if (_display_parent.has_h_scrollbar()) {
	available_height-=_horiz.h();
      }
      /* set up the min and max val */
      if (vs.min_val()!=0)
	vs.set_min_val(0);
      if (vs.max_val()!=_child_display.h()-available_height) 
	vs.set_max_val((_child_display.h()-available_height));
      /*this is not going to get used, so you can just make it
      them any value and not worry about it */
      /*
	vs.set_small_inc(1);
	*/
      if (vs.large_inc()!=available_height)
	vs.set_large_inc(available_height);
      /* set the thumb size to be a percentage */
      percentage=((double)available_height)/((double)_child_display.h());
      /* bounds check */
      if (percentage>1.0) {
	percentage=1.0;
      }
      if (vs.thumb_percentage()!=percentage)
	vs.set_thumb_percentage(percentage);
    }
  }
  /**
   * This is a utility to find out if we have (our own, not an external)
   * scrollbar.
   * @return boolean true if we have our own v scrollbar
   */
  public boolean have_v_scrollbar() {
    return (num_children()>1);
  }
  /**
   * This is a convenience function for users to call when they
   * want to know what index is at a given screen cordinate.
   * This function will return -1 if you are out of bounds.
   * @param int y the y coodinate to test
   * @return int the index of the item at a given point
   */
  public int index_for_screen_point(int y) {
    style cs=style_manager.current_style();
    int dvs=cs.drawable_vertical_space(),offset,index;

    /* short circuit out of here if empty */
    if (_child_display.num_children()==0) return -1;
    /* get the offset */
    offset=_display_parent._above_space+ dvs;
    
    /* figure out the child number at the top of box */
    index=_child_display.index_for_pixel(offset - (_child_display.y())
+ y);
    return index;
  }
} 

/**
 * This class is the helper for the listbox. It holds the 
 * listbox_child_display and the horizontal scrollbar.  It handles 
 * drawing the appropriate borders around the object.
 */
class listbox_helper extends base_parent_interactor {
  /**
   * Create a listbox helper. We default the size because we know 
   * we are going to add constraints to it.
   */
  public listbox_helper()  {
    super(0,0,10,10);
  }
  /**
   * This variable holds the amount of space to the right of the list.
   */
  protected int _right_space=3;
  /**
   * This variable holds the amount of space to the left of the list.
   */
  protected int _left_space=3;
  /**
   * This variable holds the amount of space above of the list.
   */
  protected int _above_space=3;
  /**
   * This variable holds the amount of space below of the list.
   */
  protected int _below_space=3;
  /**
   * We need to implement the border handling code, so we override
   * the drawing code to do this.
   * @param drawable d the drawable to display on
   */
  public void draw_self_local(drawable d)  {
    color_scheme cs=style_manager.default_color_scheme();
    style s=style_manager.current_style();
    int extra_bottom_space=0,height;

    /* go ahead and do the children */
    draw_children(d);

    /* if we have a bottom scrollbar, we need to not fill its area */
    if (has_h_scrollbar()) {
      /* we do, so update the extra bottom space */
      extra_bottom_space=child(2).h();
    }
    /* bevel the area right around the main child in the down direction */
    s.drawable_prepare_rect(d,_left_space,_above_space,
			    w()-(_left_space+_right_space),
			    h()-(_above_space+
				 _below_space+extra_bottom_space),false,
			    false);

    /* get the real height of the relevant area */
    height=h()-extra_bottom_space;
    /* now do our stuff ... fill the borders */
    d.setColor(cs.base());
    d.fillRect(0,0,w(),_above_space); /* top */
    d.fillRect(0,height-_below_space,w(),_below_space); /* bottom */
    /* there is now the distinct possibility that just to the right
     * or left of the horizontal scrollbar for a width of the size of the
     * right or left hand space there is a little bunch of crap left 
     * from painting the listbox_child_display... we need to make sure 
     * the user can't see this so we use h() not height
     */
    d.fillRect(w()-_right_space,0,_right_space,h()); /* right */
    d.fillRect(0,0,_left_space,h()); /* left */
  }
  /**
   * Return the amount of usable horizontal pixels in this object.
   * @return int width of the usable area
   */
  public int available_width() {
    style cs=style_manager.current_style();
    int dhs=cs.drawable_horizontal_space();
    int aw=w()-((_left_space + _right_space) +
		(2*dhs));
    return aw;
  }
  /**
   * Return the amount of usable vertical pixels in this object.
   * @return int height of the usable area
   */
  public int available_height() {
    style cs=style_manager.current_style();
    int dvs=cs.drawable_vertical_space();
    int ah=h()-((_above_space + _below_space) +
		(2*dvs));
    return ah;
  }
  /**
   * This is the pick routine for the listbox helper. If the user
   * clicks in the areas that we consider "out of bounds" - the
   * border areas, we don't allow ourselves to be picked. If the
   * user picks anywhere else, we allow it.
   * @param int pt_x the x coordinate of the point
   * @param int pt_y the y coordinate of the point
   * @param pick_collector the object to put yourself in if you are picked
   */
  public void pick(int pt_x, int pt_y, pick_collector pick_list) {
    style cs=style_manager.current_style();
    int dhs=cs.drawable_horizontal_space();
    int dvs=cs.drawable_vertical_space();
    boolean continue_traversing=false;
    int extra_bottom_space=0,height;

    /* first, test to see if we have a scrollbar... 
	 that could affect things quite a bit...*/
    if (has_h_scrollbar()) {
      /* we do, so update the extra bottom space */
      extra_bottom_space=child(2).h();
      /* we have it, is the click in its area? */
      if (pt_y>h()-child(2).h()) {
	/* it is vertically, is it horizontally?*/
	if (pt_x<=child(2).w()) {
	  continue_traversing=true;
	}
      }
    }
    /* get the real height of the relevant area */
    height=h()-extra_bottom_space;
    /* this is the case where want to enforce that you can only pick on the
       * visible area */
    if ((pt_y>=(_above_space+dvs)) &&
	(pt_y<height-(_below_space+dvs)) &&
	(pt_x>=(_left_space+dhs)) &&
	(pt_x<=w()- (_right_space+dhs))) {
      /* this will let the pick route continue */
      continue_traversing=true;
      /* now pick our parent to let it know something happened */
      pick_list.report_pick(parent());
    }
    /* now, do the deed */
    if (enabled() && visible() && continue_traversing) {
      pick_within_children(pt_x, pt_y, pick_list);
    }
  }
  /**
   * This is just a utility to find out if we have an h scrollbar
   * or not.
   */
  public boolean has_h_scrollbar() {
    return (num_children()>2);
  }
} 

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
