package sub_arctic.lib;

import sub_arctic.input.event;
/**
 * This interface is used by the menu agent code to notify interactors
 * which have popped up menus. This is used for two primary purposes:<P>
 *
 * First, two allow the object popping up the menu know when the menu
 * interaction is completed. Second, to allow the object popping the
 * menu up to implement behavior during the menu interaction which will
 * affect which menus are popped up. This is primarily intended for
 * use in menubars, which need to be able to change the currently
 * displayed menu during the menu interaction. 
 * 
 * @author Ian Smith
 */
public interface menu_notifier extends interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function gets called when the interaction with a menu ends.
   */
  public void menu_done();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called to alert the notifier that the interaction
   * is now over their area. The notifier should return true if it
   * modified the set of objects in the menu focus in response to
   * this call.
   *
   * @param int   x   the x coordinate (in the notifiers coordinate system) of 
   *                  the cursor.
   * @param int   y   the y coordinate (in the notifiers coordinate system) of 
   *                  the cursor.
   * @param event evt the event we are testing
   * @return boolean true if the notifier modified the focus set of the menu 
   *                      agent in response to this
   */
  public boolean menu_modify(int x,int y, event evt);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
