
package sub_arctic.lib;

/** 
 * Interface representing an object that transforms a set of "parameter" 
 * values from those suitable for use by a parent object into a set suitable 
 * for use by a child object (for example, during a traversal using 
 * traverse_and_collect()).
 *  
 *  @see sub_arctic.lib.interactor#traverse_and_collect
 *  @author Scott Hudson 
 */
public interface traversal_xform {
  /** Perform the transformation.
   *  
   *  @param parent_parameters an object representing parameter values as set
   *                           up for the parent object.
   *  @param child_obj         the child object that the parameter values 
   *                           should be transformed for use by (a common 
   *                           example would be to transform a point from the
   *                           parent's coordinate system into this child 
   *                           object's coordinate system).
   *  @return the resulting transformed parameters object.  Note: this can
   *          be the same object passes in.
   */
  public Object xform(
    Object     parent_parameters, 
    interactor child_obj, 
    int        child_index);

   //had:
   //*@exception sub_arctic.exception.bad_value thrown if the parent_parameters 
   //* parameter is not the proper type.
};
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
