package sub_arctic.test;

import java.applet.Applet;

/* import various pieces of the sub_arctic toolkit that we need */
import sub_arctic.lib.interactor_canvas;
import sub_arctic.lib.base_parent_interactor;
import sub_arctic.lib.button;
import sub_arctic.lib.interactor;
import sub_arctic.lib.top_level;
import sub_arctic.input.callback_object;
import sub_arctic.input.event;
import sub_arctic.constraints.std_function;
import sub_arctic.constraints.std_constraint_consts;

import java.awt.LayoutManager;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Event;

/**
 * This is a class that shows you how to use sub_arctic within an
 * AWT canvas (and an AWT interface).
 */

/*
 * NOTE: This class defaults to a 500x300 size but the width and
 * height tags in the HTML file override that.
 */

/* we inherit from the AWT applet base class */
public class hello_world_canvas extends Applet 
implements std_constraint_consts, callback_object {
  public void init() {
    interactor_canvas canvas;
    top_level top;
    Button AWTButton;

    /* build an interactor canvas and run with it */
    canvas=new interactor_canvas(500,300);
    /* put in the border layout */
    setLayout(new BorderLayout(0,0)); 
    /* put the canvas in the layout */
    add("Center",canvas);
    /* make a button */
    AWTButton=new Button("I'm an AWT button");
    add("North",AWTButton);
    /* create a toplevel for this canvas... cover the whole thing */
    top=new top_level(0,0,500,300);
    /* install the toplevel on this canvas */
    canvas.set_top_interactor(top);
    /* we are in business, roll into sub_arctic */
    pre_build_ui();
    build_ui(top);
    post_build_ui(top);
  }
  /**
   * This function is called to allow you to initialize data structures
   * that are needed for building the UI. By default, it does nothing.
   */
  public void pre_build_ui() {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called to allow you to process the now 
   * constructed interactor tree after its built.
   * 
   * @param base_parent_interactor t root of the newly constructed tree
   */
  public void post_build_ui(base_parent_interactor t) {}
  /**
   * This function gets called so you can construct your
   * user interface. The top_level provided is already installed
   * as the root of your interactor tree for this applet.
   *
   * @param base_parent_interactor top the new root interactor for this applet
   */
  public void build_ui(base_parent_interactor top) { 
    /* create a button centered in the top level interactor */
    button goodbye = new button("SubArctic Button In A Canvas", this);
    goodbye.set_x_constraint(std_function.centered(PARENT.W(), 0));
    goodbye.set_y_constraint(std_function.centered(PARENT.H(), 0));
    /* make the button a child of the top level */
    top.add_child(goodbye);
  }

  /* handle callback from the sub_arctic button. */
  public void callback(interactor from, event evt, int cb_num, 
		       Object cb_parm) {
    System.out.println("Callback from the sub_arctic button");
  }
  /* public void handle event */
  public boolean action(Event e, Object o) {
    System.out.println("Got a callback from the AWT button");
    return true;
  }
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
